"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var JOI = __importStar(require("joi"));
function serialize(joi) {
    return joi.describe();
}
exports.serialize = serialize;
function str2regex(pattern) {
    var flagLen = pattern.length - pattern.lastIndexOf("/") - 1;
    return new RegExp(pattern.substr(1, pattern.length - 2 - flagLen), pattern.substr(-flagLen));
}
var types = {
    "string": {
        create: function () { return JOI.string(); },
        inherits: "any",
        rules: {
            "regex": function (sch, arg) { return sch.regex(arg.pattern, { name: arg.name, invert: arg.invert }); },
            "min": function (sch, arg) { return sch.min(arg); },
            "max": function (sch, arg) { return sch.max(arg); },
            "token": function (sch) { return sch.token(); },
            "alphanum": function (sch) { return sch.alphanum(); },
            "base64": function (sch) { return sch.base64(); },
            "creditCard": function (sch) { return sch.creditCard(); },
            "dataUri": function (sch) { return sch.dataUri(); },
            "email": function (sch, arg) { return sch.email(arg); },
            "guid": function (sch) { return sch.guid(); },
            "hex": function (sch, arg, desc) { return sch.hex({ byteAligned: desc.flags && desc.flags.byteAligned }); },
            "hostname": function (sch) { return sch.hostname(); },
            "ip": function (sch, arg) { return sch.ip(arg); },
            "isoDate": function (sch) { return sch.isoDate(); },
            "lowercase": function (sch) { return sch.lowercase(); },
            "length": function (sch, arg) { return sch.length(arg); },
            "uuid": function (sch) { return sch.uuid(); },
            "uppercase": function (sch) { return sch.uppercase(); },
            "uri": function (sch, arg) { return sch.uri(arg); },
            "trim": function (sch, arg) { return sch.trim(arg); },
            "normalize": function (sch, arg) { return sch.normalize(arg); },
        },
        options: {},
        flags: {
            "case": function (sch, arg) { return sch; },
            "insensitive": function (sch) { return sch.insensitive(); },
            "byteAligned": function (sch) { return sch; },
            "truncate": function (sch, arg) { return sch.truncate(arg); },
            "trim": function (sch) { return sch; },
            "normalize": function (sch) { return sch; },
        }
    },
    "boolean": {
        create: function () { return JOI.boolean(); },
        inherits: "any",
        rules: {},
        options: {
            "truthy": function (sch, arg) { return sch.truthy(arg.filter(function (v) { return v !== true; })); },
            "falsy": function (sch, arg) { return sch.falsy(arg.filter(function (v) { return v !== false; })); }
        },
        flags: {
            "insensitive": function (sch, arg) { return sch.insensitive(arg); },
        }
    },
    "binary": {
        create: function () { return JOI.binary(); },
        inherits: "any",
        rules: {
            "min": function (sch, arg) { return sch.min(arg); },
            "max": function (sch, arg) { return sch.max(arg); },
            "length": function (sch, arg) { return sch.length(arg); },
        },
        options: {},
        flags: {
            "encoding": function (sch, arg) { return sch.encoding(arg); },
        }
    },
    "date": {
        create: function () { return JOI.date(); },
        inherits: "any",
        rules: {
            "min": function (sch, arg) { return sch.min(arg); },
            "max": function (sch, arg) { return sch.max(arg); },
            "greater": function (sch, arg) { return sch.greater(arg); },
            "less": function (sch, arg) { return sch.less(arg); },
        },
        options: {},
        flags: {
            "format": function (sch, arg) {
                var isoFormat = /^(?:[-+]\d{2})?(?:\d{4}(?!\d{2}\b))(?:(-?)(?:(?:0[1-9]|1[0-2])(?:\1(?:[12]\d|0[1-9]|3[01]))?|W(?:[0-4]\d|5[0-2])(?:-?[1-7])?|(?:00[1-9]|0[1-9]\d|[12]\d{2}|3(?:[0-5]\d|6[1-6])))(?![T]$|[T][\d]+Z$)(?:[T\s](?:(?:(?:[01]\d|2[0-3])(?:(:?)[0-5]\d)?|24\:?00)(?:[.,]\d+(?!:))?)(?:\2[0-5]\d(?:[.,]\d+)?)?(?:[Z]|(?:[+-])(?:[01]\d|2[0-3])(?::?[0-5]\d)?)?)?)?$/;
                if (arg.toString() != isoFormat.toString())
                    throw new Error("Unknown date format!");
                return sch.iso();
            },
            "timestamp": function (sch, arg) { return sch.timestamp(arg); },
            "multiplier": function (sch) { return sch; },
        }
    },
    "object": {
        create: function (desc) {
            if (desc.flags && desc.flags.func)
                return JOI.func();
            return JOI.object();
        },
        inherits: "any",
        rules: {
            "min": function (sch, arg, desc, opts) { return sch.min(arg); },
            "max": function (sch, arg, desc, opts) { return sch.max(arg); },
            "length": function (sch, arg, desc, opts) { return sch.length(arg); },
            "assert": function (sch, arg, desc, opts) { return sch.assert(JOI.ref(arg.ref.substr(4)), deserialize(arg.schema, opts)); },
            "type": function (sch, arg, desc, opts) { return sch.type(arg.ctor, arg.name); },
            "schema": function (sch, arg, desc, opts) { return sch.schema(); },
            // Following functions are actually for .func type
            "arity": function (sch, arg) { return sch.arity(arg); },
            "minArity": function (sch, arg) { return sch.minArity(arg); },
            "maxArity": function (sch, arg) { return sch.maxArity(arg); },
            "class": function (sch, arg) { return sch.class(); }
        },
        options: {
            "children": function (sch, arg, desc, opts) {
                var obj = {};
                for (var p in arg) {
                    obj[p] = deserialize(arg[p], opts);
                }
                return sch.keys(obj);
            },
            "patterns": function (sch, arg, desc, opts) {
                for (var _i = 0, arg_1 = arg; _i < arg_1.length; _i++) {
                    var p = arg_1[_i];
                    var pat = void 0;
                    if (p.regex)
                        pat = str2regex(p.regex);
                    sch = sch.pattern(pat, deserialize(p.rule, opts));
                }
                return sch;
            },
            "dependencies": function (sch, arg, desc, opts) {
                var _a, _b;
                for (var _i = 0, arg_2 = arg; _i < arg_2.length; _i++) {
                    var dep = arg_2[_i];
                    switch (dep.type) {
                        case 'and':
                        case 'nand':
                        case 'or':
                        case 'xor':
                        case 'oxor':
                            sch = (_a = sch)[dep.type].apply(_a, dep.peers);
                            break;
                        case 'with':
                        case 'without':
                            sch = (_b = sch)[dep.type].apply(_b, [dep.key].concat(dep.peers));
                            break;
                        default:
                            throw new Error("Unknown object dependency:" + dep.type);
                    }
                }
                return sch;
            },
            "renames": function (sch, arg, desc, opts) {
                for (var _i = 0, arg_3 = arg; _i < arg_3.length; _i++) {
                    var rn = arg_3[_i];
                    if (rn.isRegExp && typeof rn.from == "string")
                        rn.from = str2regex(rn.from);
                    sch = sch.rename(rn.from, rn.to, rn.options);
                }
                return sch;
            },
        },
        flags: {
            "allowUnknown": function (sch, arg, desc, opts) { return sch.unknown(arg); },
            "func": function (sch) { return sch; },
        }
    },
    "array": {
        create: function () { return JOI.array(); },
        inherits: "any",
        rules: {
            "min": function (sch, arg, desc, opts) { return sch.min(arg); },
            "max": function (sch, arg, desc, opts) { return sch.max(arg); },
            "length": function (sch, arg, desc, opts) { return sch.length(arg); },
            "unique": function (sch, arg, desc, opts) { return sch.unique(arg.comparator || arg.path, __assign({}, arg)); },
            "has": function (sch, arg, desc, opts) { return sch.has(deserialize(arg, opts)); }
        },
        options: {
            "items": function (sch, arg, desc, opts) {
                var _a;
                return (_a = sch).items.apply(_a, arg.map(function (a) { return deserialize(a, opts); }));
            },
            "orderedItems": function (sch, arg, desc, opts) {
                var _a;
                return (_a = sch).ordered.apply(_a, arg.map(function (a) { return deserialize(a, opts); }));
            },
        },
        flags: {
            "sparse": function (sch, arg) { return sch.sparse(arg); },
            "single": function (sch, arg) { return sch.single(arg); }
        }
    },
    "number": {
        create: function () { return JOI.number(); },
        inherits: "any",
        rules: {
            "min": function (sch, arg) { return sch.min(arg); },
            "max": function (sch, arg) { return sch.max(arg); },
            "greater": function (sch, arg) { return sch.greater(arg); },
            "less": function (sch, arg) { return sch.less(arg); },
            "integer": function (sch, arg) { return sch.integer(); },
            "precision": function (sch, arg) { return sch.precision(arg); },
            "multiple": function (sch, arg) { return sch.multiple(arg); },
            "positive": function (sch, arg) { return sch.positive(); },
            "negative": function (sch, arg) { return sch.negative(); },
            "port": function (sch, arg) { return sch.port(); }
        },
        flags: {
            "unsafe": function (sch, arg) { return sch.unsafe(arg); },
            "precision": function (sch, arg) { return sch; },
        },
        options: {}
    },
    "symbol": {
        create: function () { return JOI.symbol(); },
        inherits: "any",
        options: {
            "map": function (sch, arg, desc, opts) {
                if (!desc.flags || !desc.flags.allowOnly)
                    return sch;
                var symbolMap = opts.symbols || {};
                var arr = [];
                arg.forEach(function (k, v) {
                    arr.push([k, symbolMap[k] || v]);
                });
                return sch.map(arg);
            }
        },
        rules: {},
        flags: {}
    },
    "alternatives": {
        create: function () { return JOI.alternatives(); },
        inherits: "any",
        options: {
            "alternatives": function (sch, arg, desc, opts) {
                var _a;
                if (arg.length == 0)
                    return sch;
                arg = arg.map(function (a) { return deserialize(a, opts); });
                return (_a = sch).try.apply(_a, arg);
            },
        },
        flags: {},
        rules: {}
    },
    "any": {
        create: function () { return JOI.any(); },
        options: {
            "options": function (sch, arg) { return sch.options(arg); },
            "meta": function (sch, arg) { return sch.meta(arg); },
            "examples": function (sch, arg) { return sch.example.apply(sch, arg.map(function (ex) { return [ex.value, ex.options]; })); },
            "description": function (sch, arg) { return sch.description(arg); },
            "tags": function (sch, arg) { return sch.tags(arg); },
            "notes": function (sch, arg) { return sch.notes(arg); },
            "unit": function (sch, arg) { return sch.unit(arg); },
            "label": function (sch, arg) { return sch.label(arg); },
            "invalids": function (sch, arg) { return arg.reduce(function (prev, iv) { return prev.invalid(iv); }, sch); },
            "valids": function (sch, arg, desc) {
                if (desc.flags && desc.flags.allowOnly)
                    for (var _i = 0, arg_4 = arg; _i < arg_4.length; _i++) {
                        var valid = arg_4[_i];
                        sch = sch.only(valid);
                    }
                else
                    for (var _a = 0, arg_5 = arg; _a < arg_5.length; _a++) {
                        var valid = arg_5[_a];
                        sch = sch.allow(valid);
                    }
                return sch;
            }
        },
        flags: {
            "allowOnly": function (sch) { return sch; },
            "raw": function (sch, arg) { return sch.raw(arg); },
            "empty": function (sch, arg, desc) { return sch.empty(deserialize(arg)); },
            "error": function (sch, arg) { return sch.error(arg); },
            "presence": function (sch, arg) {
                if (arg == "required") {
                    sch = sch.required();
                }
                else if (arg == "optional") {
                    sch = sch.optional();
                }
                else if (arg == "forbidden") {
                    sch = sch.forbidden();
                }
                else {
                    throw new Error("invalid presence flag: " + arg);
                }
                return sch;
            },
            "strip": function (sch, arg) { return sch.strip(); },
            "default": function (sch, arg) { return sch.default(arg); },
        },
        rules: {}
    }
};
function getRule(to, rule) {
    var r;
    do {
        r = to.rules[rule];
        to = types[to.inherits || ""];
    } while (!r && to);
    if (!r)
        throw new Error("No rule handler found for " + rule);
    return r;
}
function getFlag(to, flag) {
    var f;
    do {
        f = to.flags[flag];
        to = types[to.inherits || ""];
    } while (!f && to);
    if (!f)
        throw new Error("No flag handler found for " + flag);
    return f;
}
function getOption(to, option) {
    var o;
    do {
        o = to.options[option];
        to = types[to.inherits || ""];
    } while (!o && to);
    if (!o)
        throw new Error("No option handler found for " + option);
    return o;
}
function deserialize(desc, opts) {
    opts = opts || {};
    var sch;
    var type = types[desc.type];
    if (!type)
        throw new Error("Unknown joi schema type: " + desc.type);
    sch = type.create(desc, opts);
    for (var op in desc) {
        if (['rules', 'flags', 'type'].indexOf(op) > -1)
            continue;
        sch = getOption(type, op)(sch, desc[op], desc, opts);
    }
    if (desc.rules)
        for (var _i = 0, _a = desc.rules; _i < _a.length; _i++) {
            var _b = _a[_i], arg = _b.arg, name_1 = _b.name;
            sch = getRule(type, name_1)(sch, arg, desc, opts);
        }
    if (desc.flags)
        for (var fl in desc.flags) {
            sch = getFlag(type, fl)(sch, desc.flags[fl], desc, opts);
        }
    return sch;
}
exports.deserialize = deserialize;
