"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var chai_1 = require("chai");
var JOI = __importStar(require("joi"));
var index_1 = require("./index");
function checkSchema(sch) {
    var ser = index_1.serialize(sch);
    var deserSch = index_1.deserialize(ser);
    var origDesc = sch.describe();
    var deserDesc = deserSch.describe();
    chai_1.expect(deserDesc).to.deep.equal(origDesc);
}
function runSchemaCheck(msg, sch) {
    it(msg, function () {
        checkSchema(sch);
    });
}
function runSingleFuncCheck(sch, funcname) {
    var _a;
    var args = [];
    for (var _i = 2; _i < arguments.length; _i++) {
        args[_i - 2] = arguments[_i];
    }
    var argName = "";
    try {
        argName += JSON.stringify(args);
    }
    catch (e) { }
    runSchemaCheck(funcname + " " + argName, (_a = sch)[funcname].apply(_a, args));
}
describe('any', function () {
    runSchemaCheck('simple any', JOI.any());
    runSingleFuncCheck(JOI.any(), 'allow', 1234);
    runSingleFuncCheck(JOI.any(), 'allow', 1234, 12345);
    runSingleFuncCheck(JOI.any(), 'allow', [1234, 12345]);
    runSingleFuncCheck(JOI.any(), 'valid', 1234);
    runSingleFuncCheck(JOI.any(), 'valid', 1234, 12345);
    runSingleFuncCheck(JOI.any(), 'valid', [1234, 12345]);
    runSingleFuncCheck(JOI.any(), 'invalid', 1234);
    runSingleFuncCheck(JOI.any(), 'invalid', 1234, 12345);
    runSingleFuncCheck(JOI.any(), 'invalid', [1234, 12345]);
    runSingleFuncCheck(JOI.any(), 'required');
    runSingleFuncCheck(JOI.any(), 'optional');
    runSingleFuncCheck(JOI.any(), 'forbidden');
    runSingleFuncCheck(JOI.any(), 'strip');
    runSingleFuncCheck(JOI.any(), 'unit', 'seconds');
    runSingleFuncCheck(JOI.any(), 'options', { convert: false });
    runSingleFuncCheck(JOI.any(), 'strict', true);
    runSingleFuncCheck(JOI.any(), 'strict', false);
    runSingleFuncCheck(JOI.any(), 'default', ['val', 'desc']);
    runSingleFuncCheck(JOI.any(), 'raw', true);
    runSingleFuncCheck(JOI.any(), 'raw', false);
    runSingleFuncCheck(JOI.any(), 'empty', '');
});
describe('string', function () {
    runSchemaCheck('simple string', JOI.string());
    runSingleFuncCheck(JOI.string(), 'regex', /1234/, { name: "hi", invert: true });
    runSingleFuncCheck(JOI.string(), 'min', 123, 'utf8');
    runSingleFuncCheck(JOI.string(), 'max', 123, 'utf8');
    runSingleFuncCheck(JOI.string(), 'token');
    runSingleFuncCheck(JOI.string(), 'alphanum');
    runSingleFuncCheck(JOI.string(), 'base64', { paddingRequired: true });
    runSingleFuncCheck(JOI.string(), 'creditCard');
    runSingleFuncCheck(JOI.string(), 'dataUri', { paddingRequired: true });
    runSingleFuncCheck(JOI.string(), 'email', { errorLevel: 2, tldWhitelist: ['de'], minDomainAtoms: 3 });
    runSingleFuncCheck(JOI.string(), 'guid');
    runSingleFuncCheck(JOI.string(), 'hex', { byteAligned: true });
    runSingleFuncCheck(JOI.string(), 'hostname');
    runSingleFuncCheck(JOI.string(), 'ip', { version: ['ipv4'], cidr: 'optional' });
    runSingleFuncCheck(JOI.string(), 'isoDate');
    runSingleFuncCheck(JOI.string(), 'length', 99, 'utf8');
    runSingleFuncCheck(JOI.string(), 'uuid');
    runSingleFuncCheck(JOI.string(), 'uri', { scheme: 'http', allowRelative: false, relativeOnly: false, allowQuerySquareBrackets: false });
    describe('converters', function () {
        runSingleFuncCheck(JOI.string(), 'lowercase');
        runSingleFuncCheck(JOI.string(), 'uppercase');
        runSingleFuncCheck(JOI.string(), 'insensitive');
        runSingleFuncCheck(JOI.string(), 'truncate', true);
        runSingleFuncCheck(JOI.string(), 'truncate', false);
        runSingleFuncCheck(JOI.string(), 'trim', true);
        runSingleFuncCheck(JOI.string(), 'trim', false);
        runSingleFuncCheck(JOI.string(), 'replace', /123/, "1234");
        runSingleFuncCheck(JOI.string(), 'normalize', 'NFD');
    });
});
describe('boolean', function () {
    runSchemaCheck('simple bool', JOI.bool());
    runSingleFuncCheck(JOI.bool(), 'truthy', 1234);
    runSingleFuncCheck(JOI.bool(), 'falsy', 1234);
    runSingleFuncCheck(JOI.bool(), 'insensitive', true);
    runSingleFuncCheck(JOI.bool(), 'insensitive', false);
});
describe('func', function () {
    runSchemaCheck('simple func', JOI.func());
    runSingleFuncCheck(JOI.func(), 'arity', 2);
    runSingleFuncCheck(JOI.func(), 'minArity', 1);
    runSingleFuncCheck(JOI.func(), 'maxArity', 3);
    runSingleFuncCheck(JOI.func(), 'class');
});
describe('number', function () {
    runSchemaCheck('simple number', JOI.number());
    runSingleFuncCheck(JOI.number(), 'unsafe', true);
    runSingleFuncCheck(JOI.number(), 'unsafe', false);
    runSingleFuncCheck(JOI.number(), 'min', 1);
    runSingleFuncCheck(JOI.number(), 'max', 3);
    runSingleFuncCheck(JOI.number(), 'greater', 1);
    runSingleFuncCheck(JOI.number(), 'less', 4);
    runSingleFuncCheck(JOI.number(), 'integer');
    runSingleFuncCheck(JOI.number(), 'precision', 2);
    runSingleFuncCheck(JOI.number(), 'multiple', 3);
    runSingleFuncCheck(JOI.number(), 'positive');
    runSingleFuncCheck(JOI.number(), 'negative');
    runSingleFuncCheck(JOI.number(), 'port');
});
describe('object', function () {
    runSchemaCheck('simple object', JOI.object());
    runSingleFuncCheck(JOI.object(), 'keys', { a: JOI.number() });
    runSingleFuncCheck(JOI.object(), 'append', { a: JOI.number() });
    runSingleFuncCheck(JOI.object(), 'min', 1);
    runSingleFuncCheck(JOI.object(), 'max', 5);
    runSingleFuncCheck(JOI.object(), 'length', 3);
    runSingleFuncCheck(JOI.object(), 'pattern', /asd/i, JOI.bool());
    runSingleFuncCheck(JOI.object(), 'and', 'a', 'b');
    runSingleFuncCheck(JOI.object(), 'nand', 'a', 'b');
    runSingleFuncCheck(JOI.object(), 'or', 'a', 'b');
    runSingleFuncCheck(JOI.object(), 'xor', 'a', 'b');
    runSingleFuncCheck(JOI.object(), 'oxor', 'a', 'b');
    runSingleFuncCheck(JOI.object(), 'with', 'a', 'b');
    runSingleFuncCheck(JOI.object(), 'with', 'a', ['b']);
    runSingleFuncCheck(JOI.object(), 'without', 'a', 'b');
    runSingleFuncCheck(JOI.object(), 'without', 'a', ['b']);
    runSingleFuncCheck(JOI.object(), 'rename', 'a', 'b');
    runSingleFuncCheck(JOI.object(), 'rename', /a/, 'b');
    runSingleFuncCheck(JOI.object(), 'rename', 'a', 'b', {
        alias: true, multiple: true, override: true, ignoreUndefined: true
    });
    runSingleFuncCheck(JOI.object(), 'assert', 'a.b', JOI.number());
    runSingleFuncCheck(JOI.object(), 'assert', JOI.ref('f.v'), JOI.number());
    runSingleFuncCheck(JOI.object(), 'assert', 'a.b', JOI.number(), 'asd');
    runSingleFuncCheck(JOI.object(), 'unknown', false);
    runSingleFuncCheck(JOI.object(), 'unknown', true);
    runSingleFuncCheck(JOI.object(), 'type', RegExp);
    runSingleFuncCheck(JOI.object(), 'type', RegExp, 'bla');
    runSingleFuncCheck(JOI.object(), 'schema');
    runSingleFuncCheck(JOI.object({ a: JOI.number() }), 'requiredKeys', 'a');
    runSingleFuncCheck(JOI.object({ a: JOI.number() }), 'optionalKeys', 'a');
    runSingleFuncCheck(JOI.object({ a: JOI.number() }), 'forbiddenKeys', 'a');
});
describe('array', function () {
    runSchemaCheck('simple array', JOI.array());
    runSingleFuncCheck(JOI.array(), 'sparse', true);
    runSingleFuncCheck(JOI.array(), 'sparse', false);
    runSingleFuncCheck(JOI.array(), 'single', true);
    runSingleFuncCheck(JOI.array(), 'single', false);
    runSingleFuncCheck(JOI.array(), 'items', JOI.number());
    runSingleFuncCheck(JOI.array(), 'ordered', JOI.number(), JOI.string());
    runSingleFuncCheck(JOI.array(), 'min', 0);
    runSingleFuncCheck(JOI.array(), 'max', 4);
    runSingleFuncCheck(JOI.array(), 'length', 3);
    runSingleFuncCheck(JOI.array(), 'unique', function (a, b) { return a - b; }, { ignoreUndefined: true });
    runSingleFuncCheck(JOI.array(), 'unique', '1');
    runSingleFuncCheck(JOI.array(), 'has', JOI.number());
});
describe('symbol', function () {
    runSchemaCheck('simple symbol', JOI.symbol());
    runSingleFuncCheck(JOI.symbol(), 'map', [
        ['one', Symbol('one')]
    ]);
});
describe('binary', function () {
    runSchemaCheck('simple binary', JOI.binary());
    runSingleFuncCheck(JOI.binary(), 'encoding', 'base64');
    runSingleFuncCheck(JOI.binary(), 'min', 5);
    runSingleFuncCheck(JOI.binary(), 'max', 10);
    runSingleFuncCheck(JOI.binary(), 'length', 7);
});
describe('date', function () {
    runSchemaCheck('simple date', JOI.date());
    runSingleFuncCheck(JOI.date(), 'min', 'now');
    runSingleFuncCheck(JOI.date(), 'min', '1-1-2000');
    runSingleFuncCheck(JOI.date(), 'max', 'now');
    runSingleFuncCheck(JOI.date(), 'max', '1-1-2020');
    runSingleFuncCheck(JOI.date(), 'greater', 'now');
    runSingleFuncCheck(JOI.date(), 'greater', '1-1-2000');
    runSingleFuncCheck(JOI.date(), 'less', 'now');
    runSingleFuncCheck(JOI.date(), 'less', '1-1-2020');
    runSingleFuncCheck(JOI.date(), 'iso');
    runSingleFuncCheck(JOI.date(), 'timestamp');
    runSingleFuncCheck(JOI.date(), 'timestamp', 'javascript');
    runSingleFuncCheck(JOI.date(), 'timestamp', 'unix');
});
describe('meta', function () {
    runSingleFuncCheck(JOI.any(), 'meta', { my: "meta object" });
    runSingleFuncCheck(JOI.any(), 'description', "my description");
    runSingleFuncCheck(JOI.any(), 'error', new Error("my err msg"));
    runSingleFuncCheck(JOI.any(), 'example', 3, { parent: { sibling: 10 } });
    runSingleFuncCheck(JOI.any(), 'example', "1234");
    runSingleFuncCheck(JOI.any(), 'example', [{ value: 'example' }]);
    runSingleFuncCheck(JOI.any(), 'notes', "my notes");
    runSingleFuncCheck(JOI.any(), 'tags', ["my tag"]);
    runSingleFuncCheck(JOI.any(), 'unit', "my unit");
    runSingleFuncCheck(JOI.any(), 'label', "my label");
    runSingleFuncCheck(JOI.any(), 'notes', ['bla', 'bla2']);
});
describe('alternatives', function () {
    checkSchema(JOI.alternatives());
    runSchemaCheck('empty', JOI.alternatives());
    runSchemaCheck('filled', JOI.alternatives(JOI.string(), JOI.number()));
});
